package com.ruoyi.busi.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.busi.domain.BusiUserLove;
import com.ruoyi.busi.service.IBusiUserLoveService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 用户关注Controller
 *
 * @author 朱春云
 * @date 2025-03-29
 */
@RestController
@RequestMapping("/busi/love")
public class BusiUserLoveController extends BaseController
{
    @Autowired
    private IBusiUserLoveService busiUserLoveService;

    /**
     * 查询用户关注列表
     */
    @PreAuthorize("@ss.hasPermi('busi:love:list')")
    @GetMapping("/list")
    public AjaxResult list(BusiUserLove busiUserLove,
                                @RequestParam(name = "pageNum", defaultValue = "1") Integer pageNum,
                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize)
    {
        Page<BusiUserLove> page = new Page<>(pageNum, pageSize);
        IPage<BusiUserLove> list = busiUserLoveService.queryListPage(busiUserLove,page);
        return success(list);
    }

    /**
     * 导出用户关注列表
     */
    @PreAuthorize("@ss.hasPermi('busi:love:export')")
    @Log(title = "用户关注", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BusiUserLove busiUserLove)
    {
        List<BusiUserLove> list = busiUserLoveService.list();
        ExcelUtil<BusiUserLove> util = new ExcelUtil<BusiUserLove>(BusiUserLove.class);
        util.exportExcel(response, list, "用户关注数据");
    }

    /**
     * 获取用户关注详细信息
     */
    @PreAuthorize("@ss.hasPermi('busi:love:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return success(busiUserLoveService.getById(id));
    }

    /**
     * 新增用户关注
     */
    @PreAuthorize("@ss.hasPermi('busi:love:add')")
    @Log(title = "用户关注", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BusiUserLove busiUserLove)
    {
        return toAjax(busiUserLoveService.save(busiUserLove));
    }

    /**
     * 修改用户关注
     */
    @PreAuthorize("@ss.hasPermi('busi:love:edit')")
    @Log(title = "用户关注", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BusiUserLove busiUserLove)
    {
        return toAjax(busiUserLoveService.updateById(busiUserLove));
    }

    /**
     * 删除用户关注
     */
    @PreAuthorize("@ss.hasPermi('busi:love:remove')")
    @Log(title = "用户关注", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        List<String> list = new ArrayList<>(Arrays.asList(ids));
        return toAjax(busiUserLoveService.removeByIds(list));
    }

    /**
     * 用户关注或取消关注接口
     * @param userId 通告主id isLove 0取消关注 1 关注
     */
    @PostMapping("/userLoveIs")
    public AjaxResult userLoveIs(Long userId,String isLove)
    {
        busiUserLoveService.userLoveIs(userId,isLove);
        return success();
    }
}
